// ------|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
/**
 * @file RMatrix.h
 * @author rand huso
 * @version 1.0
 *
 * created 12 September 2003
 * purpose template to implement matrix operations
 */
// ------|---------|---------|---------|---------|---------|---------|-------80|---------|------100|

#ifndef _RMATRIXH_
#define _RMATRIXH_

#include <iostream.h>
#include "RVector.h"

template< typename T, T * ZERO, T * ONE >
class RMatrix {

    public:

		/** exported vector definition - used as return argument */
        typedef RVector< T, ZERO > RV;

		/**
		 * Constructor initializes a dummy object with no content.
		 */

        RMatrix() {
            InitObject();
        }

		/**
		 * Constructor initializes an object to the input values.
		 *
		 * @param rows is the number of rows in the matrix
		 * @param cols is the number of columns in the matrix
		 * @param fill is the value to set for each component of the matrix
		 */

        RMatrix( const int rows, const int cols, const T & fill = *ZERO ) {
            InitObject();
            InitAndLoad( rows, cols, fill );
        }

		/**
		 * Constructor (copy constructor) to make an object from an existing object.
		 *
		 * @param right is the existing object to copy.
		 */

        RMatrix( const RMatrix & right ) {
            InitObject();
            CopyFromRight( right );
        }

		/**
		 * Destructor removes allocated memory.
		 */

        virtual ~RMatrix() {
            DeleteObject();
        }

		/**
		 * Assignment operator to copy the object and from an existing object
		 * without losing memory.
		 *
		 * @param right is the object from which to copy
		 * @return the object as copied from the right - reference for chaining assignment
		 */

        RMatrix & operator = ( const RMatrix & right ) {
            if( this == &right ) return *this;
            DeleteObject();
            CopyFromRight( right );
            return *this;
        }

		/**
		 * Stream output function to display the contents of the object.
		 *
		 * This will display the contents of the RVector components of the object.
		 * @todo the stream output should be in a format that could be read back in again.
		 *
		 * @param os is the stream operator (eg, cout, cerr, clog)
		 * @param right is the object to display
		 * @return the stream for chained output
		 */

        friend ostream & operator << ( ostream & os, RMatrix< T, ZERO, ONE > & right ) {
            os << "[" << right.m_Rows << ":";
            if( NULL != right.m_Matrix ) {
                os << endl;
                for( int row=0; row<right.m_Rows; ++row ) {
//                    int cols = right.operator[](0).Length();
//                    for( int col=0; col<cols; ++col ) {
//                        os << right[ row ][ col ];
//                        if( col+1 < cols ) os << " ";
//                    }
                    os << right.operator []( row );
                    if( row+1 < right.m_Rows ) os << endl;
                }
            } else {
                os << "null";
            }
            os << "]";
            return os;
        }

		/**
		 * Component reference for a row in the matrix.
		 *
		 * Returns the reference of a row of the matrix which may be changed as necessary.
		 *
		 * @param row is the designated component to access.
		 * @return the RVector component that makes up this row of the matrix.
		 *
		 * @todo some range checking could be inserted to make sure references are within tolerance.
		 */

        RV & operator [] ( int row ) const {
            return m_Matrix[ row ];
        }

		/**
		 * Get the number of rows in this matrix.
		 *
		 * @return the defined number of rows of the matrix.
		 */

        int rows() const {
            return m_Rows;
        }

		/**
		 * Get the number of columns in the matrix.
		 *
		 * @return the number of columns defined for the 0th row of the matrix.
		 */

        int cols() const {
            int response = 0;
            if( NULL != m_Matrix ) {
                response = m_Matrix[ 0 ].Length();
            }
            return response;
        }

		/**
		 * Mathematical addition operator.
		 *
		 * Uses the services provided by the in-place addition operator.
		 *
		 * @param right is the second matrix
		 * @return The addition of the two matrices.
		 */

        RMatrix operator + ( const RMatrix & right ) const {
            RMatrix answer( *this );
            return ( answer += right );
        }

		/**
		 * Mathematical in-place addition operator.
		 *
		 * Adds two matrices together and returns the result. The result will be the size of the
		 * current matrix, it will not expand to accommodate larger sizes. The right matrix
		 * elements that are within the bounds of the current matrix will be added, elements
		 * outside the allowable size are ignored.
		 *
		 * @param right is the second matrix
		 * @return The addition of the two matrices.
		 */

        RMatrix & operator += ( const RMatrix & right ) {
            int rows = MinRows( right );
            for( int row=0; row<rows; ++row ) {
                operator[]( row ) += right.m_Matrix[ row ];
            }
            return *this;
        }

		/**
		 * Mathematical negation operator/
		 *
		 * Uses the services provided by the in-place negation operator.
		 *
		 * @param right is the second matrix.
		 * @return the current object less the right object.
		 */

        RMatrix operator - ( const RMatrix & right ) const {
            RMatrix answer( *this );
            return ( answer -= right );
        }

		/**
		 * Mathematical in-place negation operator/
		 *
		 * Subtracts two matrices together and returns the result. The result will be the size of the
		 * current matrix, it will not expand to accommodate larger sizes. The right matrix
		 * elements that are within the bounds of the current matrix will be subtracted, elements
		 * outside the allowable size are ignored.
		 *
		 * @param right is the second matrix.
		 * @return the current object less the right object.
		 */

        RMatrix & operator -= ( const RMatrix & right ) {
            int rows = MinRows( right );
            for( int row=0; row<rows; ++row ) {
                operator[]( row ) -= right.m_Matrix[ row ];
            }
            return *this;
        }

		/**
		 * Mathematical dot product operator for two matrices.
		 *
		 * Elements of the two matrices are individually multiplied by each other. Uses
		 * the services provided by the in-place dot product operator.
		 *
		 * @param right is the second matrix
		 * @return the current object dotted into the right object
		 */

        RMatrix operator & ( const RMatrix & right ) const {
            RMatrix answer( *this );
            return ( answer &= right );
        }

		/**
		 * Mathematical in-place dot product operator for two matrices.
		 *
		 * Elements of the two matrices are individually multiplied by each other.
		 * Underlapping leaves the current matrix as is, overlapping components are ignored.
		 *
		 * @param right is the second matrix
		 * @return the current object dotted into the right object
		 */

        RMatrix & operator &= ( const RMatrix & right ) {
            int rows = MinRows( right );
            for( int row=0; row<rows; ++row ) {
                operator[]( row ) &= right.m_Matrix[ row ];
            }
            return *this;
        }

		/**
		 * Mathematical multiplication operator.
		 *
		 * Multiplies each component of the matrix times this base object.
		 *
		 * @param right is the factor to multiply times each component of the matrix.
		 * @return the matrix multiplied by the right value.
		 */

        RMatrix operator * ( const T right ) const {
            RMatrix answer( *this );
            return ( answer *= right );
        }

		/**
		 * Mathematical multiplication operator.
		 *
		 * Multiplies each component of the matrix times this base object. This method
		 * allows the base object to be on the left of the operator.
		 *
		 * @param right is the matrix for the operation
		 * @param left is the value to multiply times each component of the matrix
		 * @return the matrix multiplied by the right value.
		 */

        friend RMatrix operator * ( const T & left, const RMatrix< T, ZERO, ONE > & right ) {
            RMatrix answer( right );
            return answer *= left;
        }

		/**
		 * Mathematical in-place multiplication operator.
		 *
		 * @param right is the factor to multiply times each component of the matrix.
		 * @return the matrix multiplied by the right value.
		 */

        RMatrix operator *= ( const T right ) {
            for( int row=0; row<m_Rows; ++row ) {
                operator[]( row ) *= right;
            }
            return *this;
        }

		/**
		 * Mathematical division operator.
		 *
		 * Divides the current object (each component) by the value on the right. Uses the
		 * services provided by the in-place division operator.
		 *
		 * @param right is the factor to divide into each component of the matrix.
		 * @return the matrix divided by the right value.
		 */

        RMatrix operator / ( const T right ) const {
            RMatrix answer( *this );
            return ( answer /= right );
        }

		/**
		 * Mathematical in-place division operator.
		 *
		 * Divides each component of each row by the right value.
		 *
		 * @param right is the factor to divide into each component of the matrix.
		 * @return the matrix divided by the right value.
		 */

        RMatrix operator /= ( const T right ) {
            for( int row=0; row<m_Rows; ++row ) {
                operator[]( row ) /= right;
            }
            return *this;
        }

		/**
		 * Mathematical unary negation operator.
		 *
		 * Makes a negated copy of the original object (so it doesn't become changed).
		 *
		 * @return the copy of the object with each component negated
		 */

        RMatrix operator - () const {
            RMatrix answer( *this );
            for( int row=0; row<m_Rows; ++row ) {
                answer[ row ] = -m_Matrix[ row ];
            }
            return answer;
        }

		/**
		 * Mathematical multiplication of matrix times vector operator.
		 *
		 * Multiplies the matrix times the vector to produce the resultant vector object. Size of the
		 * resultant vector is the number of rows in the matrix. If the matrix columns are larger than
		 * the length of the right vector, additional components aren't used. The same holds for values
		 * in the vector that exceed the number of rows in the matrix.
		 *
		 * @param right is the vector for multiplication
		 * @return the mathematical result of the multiplication.
		 */

        RV operator * ( const RV & right ) const {
            RV answer( m_Rows, *ZERO );
            for( int row=0; row<m_Rows; ++row ) {
                answer[ row ] = m_Matrix[ row ] * right;
            }
            return answer;
        }

		/**
		 * Logical matrix comparison.
		 *
		 * Compares two matrices element by element to see if their components are equivalent.
		 *
		 * @param right is the matrix for comparison
		 * @return the boolean result of the comparison
		 */

        bool operator == ( const RMatrix & right ) const {
            bool response = false;
            if( m_Rows == right.m_Rows ) {
                response = true;
                for( int row=0; row<m_Rows; ++row ) {
                    if( m_Matrix[ row ] != right.m_Matrix[ row ]) {
                        response = false;
                        break;
                    }
                }
            }
            return response;
        }

		/**
		 * Logical matrix comparison.
		 *
		 * Compares two matrices element by element to see if their components are different.
		 *
		 * @param right is the matrix for comparison
		 * @return the boolean result of the comparison
		 */

        bool operator != ( const RMatrix & right ) const {
            return !( *this==right );
        }

		/**
		 * Mathematical operator to reduce the bandwidth of a matrix.
		 *
		 * This method reduces a sparse matrix and returns the numbering of input
		 * components to achieve this banding. It may be advantageous to run this
		 * twice, though sample cases haven't shown the need.
		 * (Algorithm WAS the intellectual property of Rand C. Huso, 1991)
		 *
		 * @return the vector indicating the numbering required to achieve a minimum banding
		 */

        RV Reduce() {
            RV answer = InitAnswer();
            int m_Cols = m_Matrix[ 0 ].Length();
            if( m_Rows == m_Cols ) {
                // first pass:
                for( int col=0; col<m_Cols-1; ++col ) {
                    for( int rowData=m_Rows-1; rowData>col; --rowData ) {
                        if( *ZERO != m_Matrix[ rowData ][ col ]) {
                            for( int rowEmpty=rowData-1; rowEmpty>col; --rowEmpty ) {
                                if( *ZERO == m_Matrix[ rowEmpty ][ col ] ) {
                                    if( CanSwapRows( rowData, rowEmpty, col )) {
                                        SwapRows( rowData, rowEmpty );
                                        SwapCols( rowData, rowEmpty );
                                        AnswerSwap( answer, rowData, rowEmpty );
                                        break;
                }    }    }    }    }    }
                // second pass
                for( int row=m_Rows-1; row>0; --row ) {
                    for( int colData=0; colData<row-1; ++colData ) {
                        if( *ZERO != m_Matrix[ row ][ colData ]) {
                            for( int colEmpty=colData+1; colEmpty<row; ++colEmpty ) {
                                if( *ZERO == m_Matrix[ row ][ colEmpty ] ) {
                                    if( CanSwapCols( colData, colEmpty, row )) {
                                        SwapRows( colData, colEmpty );
                                        SwapCols( colData, colEmpty );
                                        AnswerSwap( answer, colData, colEmpty );
                                        break;
                }    }    }    }    }    }
            }
            return answer;
        }

		/**
		 * Mathematical matrix multiplication.
		 *
		 * Multiplies one matrix times another one. Uses the minimum sizes for
		 * left.cols and right.rows, and result is sized from left.rows and right.cols.
		 *
		 * @param right is the matrix to multiply
		 * @return the resultant matrix
		 */

        RMatrix operator * ( const RMatrix & right ) const {
            int inner = right.rows();
            if( inner > this->cols()) inner = this->cols();
            return Multiply( inner, right );
        }

		/**
		 * Mathematical identity method.
		 *
		 * Modifies the current matrix to make it zero everywhere except the diagonal
		 * which is set to the identity / unity value.
		 *
		 * @return the current matrix set to identity.
		 */

        RMatrix & Identity() {
            int cols = m_Matrix[ 0 ].Length();
            for( int row=0; row<m_Rows; ++row ) {
                for( int col=0; col<cols; ++col ) {
                    if( row == col)    m_Matrix[ row ][ col ] = *ONE;
                    else            m_Matrix[ row ][ col ] = *ZERO;
                }
            }
            return *this;
        }

		/**
		 * Mathematical matrix inversion.
		 *
		 * Returns the matrix inverse for the supplied matrix. This operator creates a matrix
		 * that has the first and second properties of an inverse: left and right hand inverse
		 * properties: Ax = b --> with inverse of A (A12) having the following properties:
		 *   A12 *  A  * A12 = A12
		 *    A  * A12 *  A  = A
		 * (Algorithm WAS the intellectual property of Rand C. Huso, 1989)
		 *
		 * @return the inverse of the current matrix as a matrix
		 */

        RMatrix operator ~ () const {
            int cols = operator[](0).Length();
            RMatrix SM( cols, cols );
            RMatrix TM( m_Rows, m_Rows );
            SM.Identity();
            TM.Identity();
            return Invert( cols, SM, TM );
        }

		/**
		 * Mathematical matrix inverse null space operator.
		 *
		 * Returns a matrix that when multiplied by an arbitrary vector of the proper length fits
		 * the null space of the original equation:
         * The general solution is more than just Ax=b and x=A'b, the arbitrariness of the solution
         * is obtained by taking the general inverse (with left and right side properties: 1, 2)
         * (obtained above) times A, subtracted from the Identity matrix and multiplied by an
         * arbitrary vector of the same length as the x vector.
         * x (general solution) = A12 * b + ( I - A12*A ) * z
         * z is the arbitrariness and can be anything (of the proper length). A non-singular square
		 * matrix will produce an A12 that multiplied by A gives Identity - with ( I - A12*A ) being zero,
		 * ie, the A12 will be exactly the A' (full inverse with no arbitrariness).
		 *
		 * @param A12 is the matrix supplied above implementing the inverse
		 * @return the matrix enveloping the null space of the solution to the inverse
		 */

        RMatrix ImA12A( const RMatrix & A12 ) {
            RMatrix A12A = A12 * (*this);
            RMatrix I( A12A.rows(), A12A.cols());
            return ( I.Identity() - A12A );
        }

    private:

        int        m_Rows;		/** the number of rows in the matrix */
        RV    *    m_Matrix;	/** the array of RVector components making this matrix */

		/**
		 * Initializes the object to the default values.
		 */

        void InitObject() {
            m_Rows        = 0;
            m_Matrix    = NULL;
        }

		/**
		 * Initializes the object and fills the matrix with the supplied value.
		 *
		 * @param rows is the number of rows for the matrix
		 * @param cols is the number of columns for the matrix
		 * @param fill is the value to set for each node of the matrix
		 */

        void InitAndLoad( const int rows, const int cols, const T & fill ) {
            m_Rows = rows;
            if( 0 < m_Rows ) {
                m_Matrix = new RV[ m_Rows ];
                for( int i=0; i<m_Rows; ++i ) {
                    m_Matrix[ i ] = RV( cols, fill );
                }
            }
        }

		/**
		 * Copies the components from the right side to this object
		 *
		 * @param right is the matrix from which to copy
		 */

        void CopyFromRight( const RMatrix & right ) {
            m_Rows = right.m_Rows;
            if( 0 < m_Rows ) {
                m_Matrix = new RV[ m_Rows ];
                for( int i=0; i<m_Rows; ++i ) {
                    m_Matrix[ i ] = right.m_Matrix[ i ];
                }
            }
        }

		/**
		 * Deletes the object and reclaims the memory allocated.
		 */

        void DeleteObject() {
            if( NULL != m_Matrix ) {
                delete [] m_Matrix;
                m_Matrix = NULL;
            }
            m_Rows = 0;
        }

		/**
		 * Computes the minimum sizes for operations that involve two matrices.
		 *
		 * @param right is the matrix to compare for minimum size
		 * @return the minimum number of rows in the two matrices
		 */

        int MinRows( const RMatrix & right ) const
        {
            return (m_Rows<right.m_Rows?m_Rows:right.m_Rows);
        }

		/**
		 * Initialize the answer for the bandwidth reduction algorithm.
		 *
		 * @return returns an RVector initialized to the numbering of the nodes
		 */

        RV InitAnswer() const {
            RV answer( m_Rows, *ZERO );
            T value = *ZERO;
            for( int i=0; i<m_Rows; ++i ) {
                answer[ i ] = value++;
            }
            return answer;
        }

		/**
		 * Checks to see if the rows given can be swapped.
		 *
		 * The rows should not be swapped if it leaves a zero on the pivot.
		 *
		 * @param row1 is one row under consideration
		 * @param row2 is the other row under consideration
		 * @param col1 is the column that should be checked for a zero on the pivot
		 * @return the boolean result of whether or not the swap should be done
		 */

        bool CanSwapRows( const int row1, const int row2, const int col1 ) const {
            bool response = true;
            for( int col=0; col<col1; ++col ) {
                if( *ZERO == m_Matrix[ row1 ][ col ]) {
                    if( *ZERO != m_Matrix[ row2 ][ col ]) {
                        response = false;
                        break;
                    }
                }
            }
            return response;
        }

		/**
		 * Checks to see if the columns given can be swapped.
		 *
		 * The columns should not be swapped if it leaves a zero on the pivot.
		 *
		 * @param col1 is one column under consideration
		 * @param col2 is the other column under consideration
		 * @param row1 is the column that should be checked for a zero on the pivot
		 * @return the boolean result of whether or not the swap should be done
		 */

        bool CanSwapCols( const int col1, const int col2, const int row1 ) const {
            bool response = true;
            for( int row=row1+1; row<m_Rows; ++row ) {
                if( *ZERO == m_Matrix[ row ][ col1 ]) {
                    if( *ZERO != m_Matrix[ row ][ col2 ]) {
                        response = false;
                        break;
                    }
                }
            }
            return response;
        }

		/**
		 * Swaps the input rows of this matrix.
		 *
		 * This is used by the bandwidth reduction and matrix inversion routines to swap
		 * the rows of matrices.
		 *
		 * @param row1 is the first row to swap
		 * @param row2 is the second row to swap
		 */

        void SwapRows( const int row1, const int row2 ) {
            RV V                = m_Matrix[ row1 ];
            m_Matrix[ row1 ]    = m_Matrix[ row2 ];
            m_Matrix[ row2 ]    = V;
        }

		/**
		 * Swaps the input columns of this matrix.
		 *
		 * This is used by the bandwidth reduction and matrix inversion routines to swap
		 * the columns of matrices.
		 *
		 * @param col1 is the first column to swap
		 * @param col2 is the second column to swap
		 */

        void SwapCols( const int col1, const int col2 ) {
            for( int row=0; row<m_Rows; ++row ) {
                m_Matrix[ row ].Swap( col1, col2 );
            }
        }

		/**
		 * Swaps the elements in the result RVector.
		 *
		 * This is used by the bandwidth reduction algorithm.
		 *
		 * @param answer is the vector to use to perform the swap
		 * @param row1 is the first element to swap in the vector
		 * @param row2 is the second element to swap in the vector
		 */

        void AnswerSwap( RV & answer, const int row1, const int row2 ) {
            T tempValue     = answer[ row1 ];
            answer[ row1 ]  = answer[ row2 ];
            answer[ row2 ]  = tempValue;
        }

		/**
		 * Multiplies two matrices.
		 *
		 * The resulting size of the matrix is the left matrix rows and the right matrix columns.
		 *
		 * @param inner is the minimum number of left columns and right rows
		 * @param right is the right matrix for the multiplication
		 * @return the result of the multiplication of the minimum components of the two matrices.
		 */

        RMatrix Multiply( const int inner, const RMatrix & right ) const {
            int cols = right.cols();
            RMatrix answer( m_Rows, cols );
            for( int row=0; row<m_Rows; ++row ) {
                for( int col=0; col<cols; ++col ) {
                    T result = m_Matrix[ row ][ 0 ];
                    result *= right.m_Matrix[ 0 ][ col ];
                    for( int i=1; i<inner; ++i ) {
                        T temp = m_Matrix[ row ][ i ];
                        temp *= right.m_Matrix[ i ][ col ];
                        result += temp;
                    }
                    answer[ row ][ col ] = result;
                }
            }
            return answer;
        }

		/**
		 * Matrix inversion algorithm essential operation.
		 *
		 * The matrix inverse uses augmented matrices for the column and row operations. These are:
		 *   SM is the augmented matrix for column operations,
		 *   TM is the augmented matrix for row operations.
		 * For each component on the diagonal, the matrix is reduced and a value of one placed on the
		 * pivot after each component of the row or column is divided through by the pivot value and
		 * all other elements on the same row and same column of this matrix are eliminated.
		 * Processing stops when the pivot is run through each diagonal component of the matrix or no
		 * non-zero values of the matrix are detected.
		 *
		 * @param cols is the number of columns to use in the matrix
		 * @param SM is the augmented matrix for column operations
		 * @param TM is the augmented matrix for row operations
		 * @return
		 */

        RMatrix Invert( const int cols, RMatrix & SM, RMatrix & TM ) const {
            RMatrix ReduceToI( *this );
            int maxDiagonal = ( SM.m_Rows<TM.m_Rows?SM.m_Rows:TM.m_Rows );
            for( int diag=0; diag<maxDiagonal; ++diag ) {
                // swap in a usable pivot
                if( !ReduceToI.SwapInNextPivot( diag, cols, SM, TM )) {
                    // done swapping - diagonal may be reduced
                    maxDiagonal = diag;
                    break;
                }
                // divide through by pivot
                T divisor = ReduceToI.m_Matrix[ diag ][ diag ];
                ReduceToI.m_Matrix[ diag ] /= divisor;
                TM.m_Matrix[ diag ] /= divisor;
                // set pivot to *ONE
                ReduceToI.m_Matrix[ diag ][ diag ] = *ONE;
                // reduce rows using pivot
                ReduceToI.ReduceRows( diag, cols, TM );
                // reduce cols using pivot
                ReduceToI.ReduceCols( diag, cols, SM );
            }
            return SM.Multiply( maxDiagonal, TM );
        }

		/**
		 * Reduce the rows by the amount of the pivot.
		 *
		 * @param diag is the current diagonal component
		 * @param cols is the number of columns in the matrix
		 * @param TM is the augmented matrix for row operations
		 */

        void ReduceRows( const int diag, const int cols, RMatrix & TM ) {
            for( int row=1+diag; row<m_Rows; ++row ) {
                T reduceByAmount = m_Matrix[ row ][ diag ];
                for( int col=1+diag; col<cols; ++col ) {
                    T temp = m_Matrix[ diag ][ col ];
                    temp *= reduceByAmount;
                    m_Matrix[ row ][ col ] -= temp;
                }
                m_Matrix[ row ][ diag ] = *ZERO;
                for( col=0; col<TM.m_Rows; ++col ) {
                    T temp = TM.m_Matrix[ diag ][ col ];
                    temp *= reduceByAmount;
                    TM.m_Matrix[ row ][ col ] -= temp;
                }
            }
        }

		/**
		 * Reduce the columns by the amount of the pivot.
		 *
		 * @param diag is the current diagonal component
		 * @param cols is the number of columns in the matrix
		 * @param SM is the augmented matrix for column operations
		 */

        void ReduceCols( const int diag, const int cols, RMatrix & SM ) {
            for( int col=1+diag; col<cols; ++col ) {
                T reduceByAmount = m_Matrix[ diag ][ col ];
                for( int row=1+diag; row<m_Rows; ++row ) {
                    T temp = m_Matrix[ row ][ diag ];
                    temp *= reduceByAmount;
                    m_Matrix[ row ][ col ] -= temp;
                }
                m_Matrix[ diag ][ col ] = *ZERO;
                for( row=0; row<cols; ++row ) {
                    T temp = SM.m_Matrix[ row ][ diag ];
                    temp *= reduceByAmount;
                    SM.m_Matrix[ row ][ col ] -= temp;
                }
            }
        }

		/**
		 * Swaps in the next pivot for row and column operations.
		 *
		 * @param diag is the current diagonal component
		 * @param cols is the number of columns in the matrix
		 * @param SM is the augmented matrix for column operations
		 * @param TM is the augmented matrix for row operations
		 * @return true if the pivot was swapped in, false if no more pivots are available
		 */

        bool SwapInNextPivot( const int diag, const int cols, RMatrix & SM, RMatrix & TM ) {
            bool response = true; // true if we have a pivot
            // if this is a ZERO on this spot
            if( *ZERO == m_Matrix[ diag ][ diag ]) {
                response = false;
                // search through all the rows for a non-ZERO
                for( int row=1+diag; row<m_Rows; ++row ) {
                    // if we swap here - set response true
                    if( *ZERO != m_Matrix[ row ][ diag ] ) {
                        SwapRows( diag, row );
                        TM.SwapRows( diag, row );
                        response = true;
                        break;
                    }
                }
                // failing that, (if we did) search through all the cols for a non-ZERO
                if( !response ) {
                    for( int col=1+diag; col<cols; ++col ) {
                        // if we swap here - set response true
                        if( *ZERO != m_Matrix[ diag ][ col ] ) {
                            SwapCols( diag, col );
                            SM.SwapRows( diag, col );
                            response = true;
                            break;
                        }
                    }
                }
            }
            return response;
        }
};

#endif

// ------|---------|---------|------< EOF >------|---------|---------|-------80|---------|------100|
