// ------|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
/**
 * @file RVector.h
 * @author rand huso
 * @version 1.0
 *
 * created 12 September 2003
 * purpose template class for my Vector routines - part of the RMatrix template
 */
// ------|---------|---------|---------|---------|---------|---------|-------80|---------|------100|

#ifndef _RVECTORH_
#define _RVECTORH_

#include <iostream.h>

template< typename T, T * ZERO >
class RVector {

    public:

		/**
		 * Constructor for default object creation - uses default values.
		 */

        RVector() {
            InitObject();
        }

		/**
		 * Constructor for creating a vector of the specified size.
		 *
		 * @param length is the length of the vector
		 * @param fill is the default value to set for each component of the vector
		 */

        RVector( const int length, const T & fill = *ZERO ) {
            InitObject();
            InitAndLoad( length, fill );
        }

		/**
		 * Constructor (copy constructor) to make one vector from a second vector.
		 *
		 * @param right is the vector to copy for this object
		 */

        RVector( const RVector & right ) {
            InitObject();
            CopyFromRight( right );
        }

		/**
		 * Destructor
		 */

        virtual ~RVector() {
            DeleteObject();
        }

		/**
		 * Assigns the right side to the left.
		 *
		 * @param right is the vector to deep copy to the current object
		 * @return the reference for chaining the assignment
		 */

        RVector & operator = ( const RVector & right ) {
            if( this == &right ) return *this;
            DeleteObject();
            CopyFromRight( right );
            return *this;
        }

		/**
		 * Stream output of this vector.
		 *
		 * @param os is the stream object (cerr, cout, clog, for example)
		 * @param right is the vector to print
		 * @return the stream object for chaining
		 *
		 * @todo the output should be in a format that could be read back in again
		 */

        friend ostream & operator << ( ostream & os, RVector< T, ZERO > & right ) {
            os << "[" << right.m_Elements << ":";
            if( NULL != right.m_Vector ) {
                for( int i=0; i<right.m_Elements; ++i ) {
                    os << right.operator []( i );
                    if( i+1 < right.m_Elements ) os << " ";
                }
            } else {
                os << "null";
            }
            os << "]";
            return os;
        }

		/**
		 * Component reference operator.
		 *
		 * Returns the index of the component for operations that may require an r-value. No index
		 * checking is performed. This method may throw an exception if the index is outside proper range.
		 *
		 * @param index is the component to return
		 * @return the reference for the base object at this index.
		 *
		 * @todo the index should be checked to make sure it is in the proper range
		 */

        T & operator [] ( const int index ) const {
            return m_Vector[ index ];
        }

		/**
		 * Returns the length of the current vector.
		 *
		 * @return the length of the current vector
		 */

        int Length() const {
            return m_Elements;
        }

		/**
		 * Mathematical addition of two vector objects.
		 *
		 * @param right is the second vector to add
		 * @return a new vector containing the result of the addition
		 */

        RVector operator + ( const RVector & right ) const {
            RVector answer( *this );
            return answer += right;
        }

		/**
		 * Mathematical in-place addition of two vector objects.
		 *
		 * This method adds two vectors in place being careful to use the minimum values
		 * for the sizes of the two vectors.
		 *
		 * @param right is the vector to add
		 * @return the current object with the addition of the right object
		 */

        RVector & operator += ( const RVector & right ) {
            int elts = MinElements( right );
            for( int elt=0; elt<elts; ++elt ) {
                operator[]( elt ) += right.m_Vector[ elt ];
            }
            return *this;
        }

		/**
		 * Mathematical subtraction of two vector objects.
		 *
		 * Subtracts the input vector from the current and returns a new object with
		 * the answer.
		 *
		 * @param right is the vector to subtract from the current one
		 * @return a new vector with the result of the mathematical operation
		 */

        RVector operator - ( const RVector & right ) const {
            RVector answer( *this );
            return answer -= right;
        }

		/**
		 * Mathematical in-place subtraction of two vector objects.
		 *
		 * Subtracts the input vector from the current one.
		 *
		 * @param right is the vector to subtract from the current vector
		 * @return the current vector after the subtraction
		 */

        RVector & operator -= ( const RVector & right ) {
            int elts = MinElements( right );
            for( int elt=0; elt<elts; ++elt ) {
                operator[]( elt ) -= right.m_Vector[ elt ];
            }
            return *this;
        }

		/**
		 * Mathematical division of the vector by components of another vector.
		 *
		 * Performs a 'dot' type division of the two vectors.
		 *
		 * @param right is the vector to divide into the current vector
		 * @return a new vector representing the resultant division
		 */

        RVector operator / ( const RVector & right ) const {
            RVector answer( *this );
            return answer /= right;
        }

		/**
		 * Mathematical in-place division of two vectors component by component.
		 *
		 * Performs a 'dot' type division of the two vectors.
		 *
		 * @param right is the vector to divide into the current vector
		 * @return the current vector with the result of the division
		 */

        RVector & operator /= ( const RVector & right ) {
            int elts = MinElements( right );
            for( int elt=0; elt<elts; ++elt ) {
                operator[]( elt ) /= right.m_Vector[ elt ];
            }
            return *this;
        }

		/**
		 * Mathematical multiplication of two vectors.
		 *
		 * Two vectors are multiplied "dot" product / component-wise.
		 *
		 * @param right is the vector to multiply times the current one.
		 * @return a new object containing the results of the operation
		 */

        RVector operator & ( const RVector & right ) const {
            RVector answer( *this );
            return ( answer &= right );
        }

		/**
		 * Mathematical in-place multiplication of two vectors.
		 *
		 * The two vectors are multiplied 'dot' product / component-wise.
		 *
		 * @param right is the vector to multiply times the current one
		 * @return the current object after the operation
		 */

        RVector & operator &= ( const RVector & right ) {
            int elts = MinElements( right );
            for( int elt=0; elt<elts; ++elt ) {
                operator[]( elt ) *= right.m_Vector[ elt ];
            }
            return *this;
        }

		/**
		 * Mathematical multiplication of two vectors.
		 *
		 * Returns a value comprising the dot product of this vector with the input vector and
		 * summing the resultant products.
		 *
		 * @param right is the vector to 'dot' into the current vector
		 * @return the 'dot' product of the two vectors
		 */

        T operator * ( const RVector & right ) const {
            int elts = MinElements( right );
            T temp;
            T answer( operator[]( 0 ));
            answer *= right.operator []( 0 );
            for( int elt=1; elt<elts; ++elt ) {
                temp = operator[]( elt );
                temp *= right.m_Vector[ elt ];
                answer += temp;
//                answer += ( operator[]( elt ) * right.m_Vector[ elt ]);
            }
            return answer;
        }

		/**
		 * Mathematical multiplication operator.
		 *
		 * Every component of the current vector is multiplied by the number.
		 *
		 * @param right is the value to multiply times each component
		 * @return a new vector representing the multiplication
		 */

        RVector operator * ( const T & right ) const {
            RVector answer( *this );
            return answer *= right;
        }

		/**
		 * Mathematical multiplication of a number times a vector.
		 *
		 * Multiplies each component of the vector with the input value. This allows for
		 * multiplying a value times a vector.
		 *
		 * @param left is the value to multiply times the vector
		 * @param right is the vector to be multiplied by the left value
		 * @return a new vector bearing the result of the multiplication
		 */

        friend RVector operator * ( const T & left, const RVector< T, ZERO > & right ) {
            RVector answer( right );
            return answer *= left;
        }

		/**
		 * Mathematical in-place multiplication of vector times value.
		 *
		 * The value is multiplied into each component of the vector
		 *
		 * @param right is the value to multiply to each component of the vector
		 * @return the object after the multiplication
		 */

        RVector & operator *= ( const T & right ) {
            for( int elt=0; elt<m_Elements; ++elt ) {
                operator[]( elt ) *= right;
            }
            return *this;
        }

		/**
		 * Mathematical division of the vector by a value.
		 *
		 * Each component of the vector is divided by the value.
		 *
		 * @param right is the value to divide into the vector
		 * @return a new vector bearing the result of the division
		 */

        RVector operator / ( const T & right ) const {
            RVector answer( *this );
            return answer /= right;
        }

		/**
		 * Mathematical in-place division of the vector by a value.
		 *
		 * Each component of the vector is divided by the value.
		 *
		 * @param right is the value to divide into the vector
		 * @return the current vector bearing the result of the division
		 */

        RVector & operator /= ( const T & right ) {
            for( int elt=0; elt<m_Elements; ++elt ) {
                operator[]( elt ) /= right;
            }
            return *this;
        }

		/**
		 * Mathematical negation of the current vector.
		 *
		 * The negation operator is applied to each component of the vector.
		 *
		 * @return a new vector which is the negative of the original
		 */

        RVector operator - () const {
            RVector answer( *this );
            for( int elt=0; elt<m_Elements; ++elt ) {
                answer[ elt ] = -m_Vector[ elt ];
            }
            return answer;
        }

		/**
		 * Logical equivalence operator.
		 *
		 * Compares two vectors to see if the components are equivalent.
		 *
		 * @param right is the vector to compare with the current
		 * @return boolean result of the comparison
		 */

        bool operator == ( const RVector & right ) const {
            bool response = false;
            if( m_Elements == right.m_Elements ) {
                response = true;
                for( int elt=0; elt<m_Elements; ++elt ) {
                    if( m_Vector[ elt ] != right.m_Vector[ elt ]) {
                        response = false;
                        break;
                    }
                }
            }
            return response;
        }

		/**
		 * Logical non-equivalence operator.
		 *
		 * Compares two vectors to see if the components are different.
		 *
		 * @param right is the vector to compare with the current
		 * @return boolean result of the comparison
		 */

        bool operator != ( const RVector & right ) const {
            return !( *this==right );
        }

		/**
		 * Operator to swap components in the vector.
		 *
		 * @param elt1 is one component to swap
		 * @param elt2 is the other component to swap
		 */

        void Swap( const int elt1, const int elt2 ) {
            T elt;
            if( NULL != m_Vector ) {
                elt                    = operator[]( elt2 );
                operator[]( elt2 )    = operator[]( elt1 );
                operator[]( elt1 )    = elt;
            }
        }

    private:

        int     m_Elements;		/** the number of elements in the vector */
        T   *   m_Vector;		/** the array of components making up this vector */

		/**
		 * Initialize the object to the null and zero state.
		 */

        void InitObject() {
            m_Vector    = NULL;
            m_Elements  = 0;
        }

		/**
		 * Initialize the object and set the components based on input values.
		 *
		 * @param length is the length of the desired object
		 * @param fill is the value to set for each component of the new object
		 */

        void InitAndLoad( const int length, const T & fill ) {
            m_Elements = length;
            if( 0 < m_Elements ) {
                m_Vector = new T[ m_Elements ];
                for( int i=0; i<m_Elements; ++i ) {
                    m_Vector[ i ] = fill;
                }
            }
        }

		/**
		 * Copy the vector from the right.
		 *
		 * Copies the components from the right vector to this one.
		 *
		 * @param right is the vector to mirrir
		 */

        void CopyFromRight( const RVector & right ) {
            m_Elements = right.m_Elements;
            if( 0 < m_Elements ) {
                m_Vector = new T[ m_Elements ];
                for( int i=0; i<m_Elements; ++i ) {
                    m_Vector[ i ] = right.m_Vector[ i ];
                }
            }
        }

		/**
		 * Delete the object and reclaim the memory allocated.
		 */

        void DeleteObject() {
            if( NULL != m_Vector ) {
                delete [] m_Vector;
                m_Vector = NULL;
            }
            m_Elements = 0;
        }

		/**
		 * Finds the minimum length of the current vector and the input vector.
		 *
		 * @param right is the vector with which to compare lengths
		 * @return the lesser of the lengths
		 */

        int MinElements( const RVector & right ) const {
            return (m_Elements<right.m_Elements?m_Elements:right.m_Elements);
        }
};

#endif

// ------|---------|---------|------< EOF >------|---------|---------|-------80|---------|------100|
